        .CHAP   PRODEF.ASM
        .BASE   10'10
        .LAYOUT LENGTH 0

; Patrick Faeh, December 1984 - January 1986/April 1989/June 1991/January 1992
; (c) 1984-1992 Patrick Faeh, La Colombire, CH-1783 Pensier, Switzerland

; This file contains the constant definitions for an interpreter
; table list statement.
; Word organization: high-low (high byte first)
; The interpreter table list (extension .PROcessor) contains a header
; with informations.

ASVERS  = 1       ; must be the same as for the assembler

; status
MHIGHLOW= 2**0    ; byte organisation
M2BYTES = 2**1    ; 2 bytes for the mnemonic value
MEVENAD = 2**2    ; instruction must be at even address

VAL     = 1       ; separates operand and operand value.


; FIRST word of an interpreter table list statement (test)
;  direct address of next routine in interpreter table:
;   |00|xxxxxx|xxxxxxxx|
;  operand analysis with address in interpreter table:
;   |01|xxxxxx|xxxxxxxx|
OA      = 16'4000 ; must be added to operand address
;	OpAddr+OA
;	OpAddr:	.ASCII	"A0<VAL><8>A1<VAL><9><0>"
;			 |	   |	      |
;			 |	   |	      0 = no more choices
;			 |	   A1 = 2nd choice, value = 9
;			 A0 = 1st choice, value = 8
;	or:
;	OpAddr:	.ASCII	"A<"0"+16'80><"1"+16'80><VAL><8><0>"
;			 |    |	      	 |		 |
;			 |    |	         |		 0 = no more choices
;			 |    |		 upper limit of range ("1")
;			 |    lower limit of range ("0")
;			 range A0..A1 = 1st & 2nd choice, value = 8..9
;	to be a valid operand, the character following a found choice
;	must not be a symbol name character.
;  compare text (address given):
;   |10|xxxxxx|xxxxxxxx|
CT      = 16'8000 ; must be added to text address
;	TextAddr+CT
;	TextAddr:.ASCIZ	"R16^"
;	the character following the text found may be any character.
;  main assembler routine:
;   |11|xxxxxx|xxxxxxxx|
;       !_test forward reference

;   fonction|parameter
MAROK   = 16'C000 ; ok analyze, continue correctly
MARTC   = 16'C100 ; test character (parameter)
MARTGC  = 16'C200 ; test and get character (parameter)
MARGPV  = 16'C300 ; get positif value of P (parameter) bits
MARGSV  = 16'C400 ; get signed value of P (parameter) bits
MARAND  = 16'C500 ; if (P AND EXVAL[1][LSB])=0 ?
MARGRV  = 16'C600 ; get relativ value of P (parameter) bits:
                  ; value = addr - (APC+offset)
MARTON  = 16'C700 ; test if F[P,1] = 0
MARTGS  = 16'C800 ; test and get sepa [space,tab]
MARTRE  = 16'C900 ; test if (P MOD 16) <= EXVAL[1][LSB] <= (P DIV 16)
MARTE8  = 16'CA00 ; test if EXVAL[1][LSB] = P
MARTGL  = 16'CB00 ; test and get list of registers, P.8 address, out D
MARTE16 = 16'CC00 ; test if EXVAL[1][LSB..LSB+1] = {P} (16 bit, MSB-LSB)
MARTEL  = 16'CD00 ; test if End of Line
MARTL8  = 16'CE00 ; test if EXVAL[1][LSB] < P
MARTFC  = 16'CF00 ; test forward character = P
MARTFT  = 16'D000 ; test forward text = P.8 address
MARTGCL = 16'D100 ; test and get character in limits (address P.8: >= lower
                  ; bound.8, <= higher bound.8). If in limits then
                  ; EXVAL[1][LSB+1] = character, EXVAL[1][LSB] = character
                  ; - lower bound.
MARGVL  = 16'D200 ; get value in limits, get positif value of {P.8} bits and
                  ; test if (EX1LSB >= {P.8+1}) and (EX1LSB <= {P.8+2}).
MARTCL  = 16'D300 ; test characters in limits (same as MARTGCL without get)
MARTPS  = 16'D400 ; test .PROCSET value, P.8 address, {P} = mask,
                  ; {P+1} = value, {P+2} = operation (see PRSET..)

TFR     = 16'2000 ; reject expression if forward reference (MARGPV,MARGSV,
		  ; MARGRV,MARGVL); no range allowed (MARTGL).
PRSETEQ = 0       ; (PRSETVAL .AND. mask) .EQ. value
PRSETHS = 1       ; (PRSETVAL .AND. mask) .HS. value


; 2nd...last-1 word of an interpreter table list statement (code generation)
;  |xxxxx|xxx|xxxxxxxx|
; function |  parameter P
;        load in field(i) -> Vi
;
; Syntax:
; first letter    second/third letter
;
; A Add           D Dummy (internal variable, 2 x 8 bit, no carry LSB->MSB)
; I mIrror        E Expression (EXVAL[1])
; M Move          L Length of field(i)
; N aNd           M Mnemonic value (8 or 16 bit)
; O Or            O Operand value (8 bit)
; F Function (P)  P Parameter (8 bit)
; T Transfer      S Shifted...
;                 I Index value (Vi)
;                 V Field[Vi,1]
;                 A offset APC
; or extension functions.

; general
TERM    = 0*16'800 ; TERMinate code generation
M_P_L   = 1*16'800 ; L[i] := P
M_P_I   = 2*16'800 ; replace i by P if P in [0..7] else nothing
T_V_V   = 3*16'800 ; transfer up/down F[i], P elements
; move
M_MP_V  = 4*16'800 ; F[i,1] := Mnemonic + P                (*)
M_P_V   = 5*16'800 ; F[i,1] := P
M_V_V   = 6*16'800 ; F[i,1] := F[P,1]
M_OP_V  = 7*16'800 ; F[i,1] := Operand + P
M_E_V   = 8*16'800 ; F[i,1..P] := EXVAL[1][1..P]           (+)
M_SV_D  = 9*16'800 ; D[0..1] := Shift(F[i,1],P)
M_SD_D  =10*16'800 ; D[0..1] := Shift(D[0..1],P)
M_P_D   =11*16'800 ; D[1] := 0; D[0] := P
M_SE_V  =12*16'800 ; F[i,1] := Shift(EXVAL[1][LSB],P)
M_SO_V  =13*16'800 ; F[i,1] := Shift(Operand,P)
M_D_V   =14*16'800 ; F[i,1..2] := D[0..1]                  (#)
; add
A_MP_V  =15*16'800 ; F[i,1] := F[i,1] + Mnemonic + P       (*)
A_P_V   =16*16'800 ; F[i,1] := F[i,1] + P
A_OP_V  =17*16'800 ; F[i,1] := F[i,1] + Operand + P
A_SO_V  =18*16'800 ; F[i,1] := F[i,1] + Shift(Operand,P)
A_P_D   =19*16'800 ; D[0] := D[0] + P
A_D_V   =20*16'800 ; F[i,1..2] := F[i,1..2] + D[0..1]      (#)
A_SV_D  =21*16'800 ; D[0..1] := D[0..1] + Shift(F[i,1],P)
A_SE_D  =22*16'800 ; D[0..1] := D[0..1] + Shift(EXVAL[1][LSB],P)
A_SO_D  =23*16'800 ; D[0..1] := D[0..1] + Shift(Operand,P)
A_SE_V  =24*16'800 ; F[i,1] := F[i,1] + Shift(EXVAL[1][LSB],P)
A_P_V2  =25*16'800 ; F[i,2] := F[i,2] + P
; other functions:
F_D_E   =26*16'800 ; EXVAL[1] := EXVAL[1] Function(P) D
O_P_V   =27*16'800 ; F[i,1] := F[i,1] OR P
N_P_V   =28*16'800 ; F[i,1] := F[i,1] AND P
EXTF    =29*16'800 ; extension functions (see below)
I_D_D   =30*16'800 ; D := MIRROR(D), size defined by P
M_D_A   =31*16'800 ; offset APC := D[0]

; (*)   if 16 bit/Mnemonic then: F(i,1) := Mne.high + ...; F(i,2) := Mne.low;
; (+)   byte order depend on processor philosophy and LSBFIRST/MSBFIRST
; (#)   P MOD 64 = 0, 2 bytes: CPU dependant, LSBFIRST, MSBFIRST
;       P MOD 64 = 1, 1 byte: F[i,1], D[0] (LSBFIRST), D[1] (MSBFIRST)
;       P MOD 64 = 2, 1 byte: F[i,2], D[0] (LSBFIRST), D[1] (MSBFIRST)

; values added to code generations commands above
RIGHT   = 0*(2**7) ; with S
LEFT    = 1*(2**7) ; with S
DOWN    = RIGHT    ; with T_
UP      = LEFT     ; with T_
LSBFIRST= 16'80    ; LSB first (with _E_, else processor dependent)
MSBFIRST= 16'C0    ; MSB first (")
DALL    = 0        ; with I_D_D, whole D
DLOW    = 1        ; with I_D_D, only D[0]
FADD    = 0        ; with F_D_E, function ADD
FSUB    = 1        ; with F_D_E, function SUB
FMUL    = 2        ; with F_D_E, function MUL
FAND    = 3        ; with F_D_E, function AND
FAPC    = 4        ; with F_D_E, EXVAL[1]:=APC

; field(i), added to code generations commands above
V0      = 0*16'100
V1      = 1*16'100
V2      = 2*16'100
V3      = 3*16'100
V4      = 4*16'100
V5      = 5*16'100
V6      = 6*16'100
V7      = 7*16'100

; extension functions (EXTF):
EATPTC  = EXTF+V0 ; advance text pointer till character N
		; stop when EOL (but save before actual text pointer)
ERTP    = EATPTC+16'80 ; restore text pointer (inverse to EATPTC)
EATPTCC = EXTF+V1 ; advance text pointer till character N
		; stop when EOL or comma (but save before actual text pointer)
A_SLPV_V= EXTF+16'A0 ; F[i,1] := F[i,1] + (PROCVALi[PVi] shl 0..7)
A_SLPV_D= EXTF+16'C0 ; D := D + (PROCVALi[PVi] shl 0..7)
M_SLPV_E= EXTF+16'E0 ; EXVAL[1] := (PROCVALi[PVi] shl 0..7)
; procval field, added to _SLPV_ commands above (.PROCVAL values)
PV0	= 0*16'8
PV1	= 1*16'8
PV2	= 2*16'8
PV3	= 3*16'8


; last ({INWITS}) word of an interpreter table list statement (decision)
; |xx|xx|xxxxxxxxxxxx|
;  |  |  relatif addressing (to current APC of this word) DIV 2
;  |  |  if analyze was correct
;  |  what to do if analyze was correct
;  what to do if analyze was not correct

; analyze correct
EOA     = 0*16'1000 ; end of analyze
GCJ     = 1*16'1000 ; get comma and jump
JMP     = 2*16'1000 ; jump
RETOK   = 3*16'1000 ; return OK from interpreter subroutine
; analyze was not correct
;       = 0*16'4000 ; nothing, try with next line
;       = 1*16'4000 ; -
ILOP    = 2*16'4000 ; illegal operands
RETKO   = 3*16'4000 ; return KO from interpreter subroutine

; separator:    (space|tab)
; comma:        [separator]","[separator]


        .END

